@@ -20,6 +20,8 @@
 /**
  * SECTION:element-dfbvideosink
  *
+ * <refsect2>
+ * <para>
  * DfbVideoSink renders video frames using the
  * <ulink url="http://www.directfb.org/">DirectFB</ulink> library.
  * Rendering can happen in two different modes :
@@ -45,19 +47,19 @@
  * <listitem>
  *   <para>
  *   Embedded: this mode will render video frames in a 
- *   #GstDfbVideoSink:surface provided by the
+ *   <link linkend="GstDfbVideoSink--surface">surface</link> provided by the
  *   application developer. This is a more advanced usage of the element and
  *   it is required to integrate video playback in existing 
  *   <ulink url="http://www.directfb.org/">DirectFB</ulink> applications.
  *   </para>
  *   <para>
  *   When using this mode the element just renders to the
- *   #GstDfbVideoSink:surface provided by the 
+ *   <link linkend="GstDfbVideoSink--surface">surface</link> provided by the 
  *   application, that means it won't handle navigation events and won't resize
- *   the #GstDfbVideoSink:surface to fit video
+ *   the <link linkend="GstDfbVideoSink--surface">surface</link> to fit video
  *   frames geometry. Application has to implement the necessary code to grab
  *   informations about the negotiated geometry and resize there
- *   #GstDfbVideoSink:surface accordingly.
+ *   <link linkend="GstDfbVideoSink--surface">surface</link> accordingly.
  *   </para>
  * </listitem>
  * </itemizedlist>
@@ -65,19 +67,20 @@
  * optimize memory allocation time and handle reverse negotiation. Indeed if 
  * you insert an element like videoscale in the pipeline the video sink will
  * negotiate with it to try get a scaled video for either the fullscreen layout
- * or the application provided external #GstDfbVideoSink:surface.
- *
- * <refsect2>
+ * or the application provided external
+ * <link linkend="GstDfbVideoSink--surface">surface</link>.
+ * </para>
  * <title>Example application</title>
  * <para>
  * <include xmlns="http://www.w3.org/2003/XInclude" href="element-dfb-example.xml" />
  * </para>
- * </refsect2>
- * <refsect2>
  * <title>Example pipelines</title>
- * |[
+ * <para>
+ * Here is a test pipeline to test the colorbalance interface :
+ * <programlisting>
  * gst-launch -v videotestsrc ! dfbvideosink hue=20000 saturation=40000 brightness=25000
- * ]| test the colorbalance interface implementation in dfbvideosink
+ * </programlisting>
+ * </para>
  * </refsect2>
  */
 
@@ -93,12 +96,18 @@
 #include "dfbvideosink.h"
 
 #include <string.h>
-#include <stdlib.h>
 
 /* Debugging category */
 GST_DEBUG_CATEGORY_STATIC (dfbvideosink_debug);
 #define GST_CAT_DEFAULT dfbvideosink_debug
 
+/* ElementFactory information */
+static const GstElementDetails gst_dfbvideosink_details =
+GST_ELEMENT_DETAILS ("DirectFB video sink",
+    "Sink/Video",
+    "A DirectFB based videosink",
+    "Julien Moutte <julien@moutte.net>");
+
 /* Default template */
 static GstStaticPadTemplate gst_dfbvideosink_sink_template_factory =
     GST_STATIC_PAD_TEMPLATE ("sink",
@@ -135,9 +144,7 @@ static void gst_dfbvideosink_surface_destroy (GstDfbVideoSink * dfbvideosink,
     GstDfbSurface * surface);
 
 static GstVideoSinkClass *parent_class = NULL;
-static GstBufferClass *surface_parent_class = NULL;
 
-#ifndef GST_DISABLE_GST_DEBUG
 static const char *
 gst_dfbvideosink_get_format_name (DFBSurfacePixelFormat format)
 {
@@ -186,7 +193,6 @@ gst_dfbvideosink_get_format_name (DFBSurfacePixelFormat format)
       return "UNKNOWN";
   }
 }
-#endif /* GST_DISABLE_GST_DEBUG */
 
 /* Creates miniobject and our internal surface */
 static GstDfbSurface *
@@ -263,8 +269,8 @@ gst_dfbvideosink_surface_create (GstDfbVideoSink * dfbvideosink, GstCaps * caps,
      or we are going to run into serious stride issues */
   if (GST_BUFFER_SIZE (surface) != size) {
     GST_WARNING_OBJECT (dfbvideosink, "DirectFB surface size (%dx%d=%d) "
-        "differs from GStreamer requested size %u", pitch, surface->height,
-        GST_BUFFER_SIZE (surface), (guint) size);
+        "differs from GStreamer requested size %d", pitch, surface->height,
+        GST_BUFFER_SIZE (surface), size);
     goto fallback;
   }
 
@@ -291,8 +297,8 @@ fallback:
     surface->surface->Release (surface->surface);
     surface->surface = NULL;
   }
-  GST_DEBUG_OBJECT (dfbvideosink, "allocating a buffer (%p) of %u bytes",
-      surface, (guint) size);
+  GST_DEBUG_OBJECT (dfbvideosink, "allocating a buffer (%p) of %d bytes",
+      surface, size);
 
   succeeded = TRUE;
 
@@ -322,14 +328,18 @@ gst_dfbvideosink_surface_destroy (GstDfbVideoSink * dfbvideosink,
     surface->surface = NULL;
   }
 
+  if (GST_BUFFER (surface)->malloc_data) {
+    g_free (GST_BUFFER (surface)->malloc_data);
+    GST_BUFFER (surface)->malloc_data = NULL;
+  }
+
   if (surface->dfbvideosink) {
     /* Release the ref to our sink */
     surface->dfbvideosink = NULL;
     gst_object_unref (dfbvideosink);
   }
 
-  GST_MINI_OBJECT_CLASS (surface_parent_class)->finalize (GST_MINI_OBJECT
-      (surface));
+  return;
 }
 
 static gpointer
@@ -339,8 +349,8 @@ gst_dfbvideosink_event_thread (GstDfbVideoSink * dfbvideosink)
 
   while (dfbvideosink->running) {
     /* Wait for an event with a 50 ms timeout */
-    dfbvideosink->event_buffer->
-        WaitForEventWithTimeout (dfbvideosink->event_buffer, 0, 50);
+    dfbvideosink->event_buffer->WaitForEventWithTimeout (dfbvideosink->
+        event_buffer, 0, 50);
 
     /* Do we have an event ? */
     ret = dfbvideosink->event_buffer->HasEvent (dfbvideosink->event_buffer);
@@ -574,14 +584,10 @@ gst_dfbvideosink_setup (GstDfbVideoSink * dfbvideosink)
         "DirectFB fullscreen");
     if (!dfbvideosink->dfb) {
       DFBGraphicsDeviceDescription hw_caps;
-      char *argv[] = { (char *) "-", (char *) "--dfb:quiet", NULL };
-      int argc = 2;
-      char **args;
 
       GST_DEBUG_OBJECT (dfbvideosink, "initializing DirectFB");
 
-      args = argv;
-      ret = DirectFBInit (&argc, &args);
+      ret = DirectFBInit (0, NULL);
 
       if (ret != DFB_OK) {
         GST_WARNING_OBJECT (dfbvideosink, "DirectFB initialization failed");
@@ -667,16 +673,16 @@ gst_dfbvideosink_setup (GstDfbVideoSink * dfbvideosink)
 
       /* Check that this layer is able to do colorbalance settings */
       if (dl_desc.caps & DLCAPS_BRIGHTNESS) {
-        channels_list = g_list_append (channels_list, (char *) "BRIGHTNESS");
+        channels_list = g_list_append (channels_list, "BRIGHTNESS");
       }
       if (dl_desc.caps & DLCAPS_CONTRAST) {
-        channels_list = g_list_append (channels_list, (char *) "CONTRAST");
+        channels_list = g_list_append (channels_list, "CONTRAST");
       }
       if (dl_desc.caps & DLCAPS_HUE) {
-        channels_list = g_list_append (channels_list, (char *) "HUE");
+        channels_list = g_list_append (channels_list, "HUE");
       }
       if (dl_desc.caps & DLCAPS_SATURATION) {
-        channels_list = g_list_append (channels_list, (char *) "SATURATION");
+        channels_list = g_list_append (channels_list, "SATURATION");
       }
 
       if (channels_list) {
@@ -687,7 +693,7 @@ gst_dfbvideosink_setup (GstDfbVideoSink * dfbvideosink)
           GstColorBalanceChannel *channel = NULL;
 
           GST_DEBUG_OBJECT (dfbvideosink, "adding %s as a colorbalance channel",
-              (const char *) walk->data);
+              walk->data);
 
           channel = g_object_new (GST_TYPE_COLOR_BALANCE_CHANNEL, NULL);
           channel->label = g_strdup (walk->data);
@@ -1534,7 +1540,7 @@ gst_dfbvideosink_show_frame (GstBaseSink * bsink, GstBuffer * buf)
 
   if (mem_cpy) {
     IDirectFBSurface *dest = NULL, *surface = NULL;
-    guint8 *data;
+    gpointer data;
     gint dest_pitch, src_pitch, line;
     GstStructure *structure;
 
@@ -1567,9 +1573,7 @@ gst_dfbvideosink_show_frame (GstBaseSink * bsink, GstBuffer * buf)
     /* Center / Clip */
     gst_video_sink_center_rect (src, dst, &result, FALSE);
 
-    res =
-        surface->GetSubSurface (surface, (DFBRectangle *) (void *) &result,
-        &dest);
+    res = surface->GetSubSurface (surface, (DFBRectangle *) & result, &dest);
     if (res != DFB_OK) {
       GST_WARNING_OBJECT (dfbvideosink, "failed when getting a sub surface");
       ret = GST_FLOW_UNEXPECTED;
@@ -1581,7 +1585,7 @@ gst_dfbvideosink_show_frame (GstBaseSink * bsink, GstBuffer * buf)
       dfbvideosink->layer->WaitForSync (dfbvideosink->layer);
     }
 
-    res = dest->Lock (dest, DSLF_WRITE, (void *) &data, &dest_pitch);
+    res = dest->Lock (dest, DSLF_WRITE, &data, &dest_pitch);
     if (res != DFB_OK) {
       GST_WARNING_OBJECT (dfbvideosink, "failed locking the external "
           "subsurface for writing");
@@ -1638,7 +1642,7 @@ gst_dfbvideosink_show_frame (GstBaseSink * bsink, GstBuffer * buf)
 
     if (dfbvideosink->hw_scaling) {
       dfbvideosink->primary->StretchBlit (dfbvideosink->primary,
-          surface->surface, NULL, (DFBRectangle *) (void *) &result);
+          surface->surface, NULL, (DFBRectangle *) & result);
     } else {
       DFBRectangle clip;
 
@@ -1696,8 +1700,8 @@ gst_dfbvideosink_buffer_alloc (GstBaseSink * bsink, guint64 offset, guint size,
 
   dfbvideosink = GST_DFBVIDEOSINK (bsink);
 
-  GST_LOG_OBJECT (dfbvideosink, "a buffer of %u bytes was requested with caps "
-      "%" GST_PTR_FORMAT " and offset %" G_GUINT64_FORMAT, size, caps, offset);
+  GST_DEBUG_OBJECT (dfbvideosink, "a buffer of %d bytes was requested "
+      "with caps %" GST_PTR_FORMAT " and offset %llu", size, caps, offset);
 
   if (G_UNLIKELY (!dfbvideosink->setup)) {
     GST_DEBUG_OBJECT (dfbvideosink, "we are not setup yet, can't allocate!");
@@ -1887,8 +1891,6 @@ gst_dfbsurface_class_init (gpointer g_class, gpointer class_data)
 {
   GstMiniObjectClass *mini_object_class = GST_MINI_OBJECT_CLASS (g_class);
 
-  surface_parent_class = g_type_class_peek_parent (g_class);
-
   mini_object_class->finalize = (GstMiniObjectFinalizeFunction)
       gst_dfbsurface_finalize;
 }
@@ -1940,7 +1942,8 @@ gst_dfbvideosink_navigation_send_event (GstNavigation * navigation,
   GstDfbVideoSink *dfbvideosink = GST_DFBVIDEOSINK (navigation);
   GstEvent *event;
   GstVideoRectangle src, dst, result;
-  double x, y, old_x, old_y;
+  gint width, height;
+  double x, y;
   GstPad *pad = NULL;
 
   src.w = GST_VIDEO_SINK_WIDTH (dfbvideosink);
@@ -1954,8 +1957,8 @@ gst_dfbvideosink_navigation_send_event (GstNavigation * navigation,
   /* Our coordinates can be wrong here if we centered the video */
 
   /* Converting pointer coordinates to the non scaled geometry */
-  if (gst_structure_get_double (structure, "pointer_x", &old_x)) {
-    x = old_x;
+  if (gst_structure_get_double (structure, "pointer_x", &x)) {
+    double old_x = x;
 
     if (x >= result.x && x <= (result.x + result.w)) {
       x -= result.x;
@@ -1968,8 +1971,8 @@ gst_dfbvideosink_navigation_send_event (GstNavigation * navigation,
         "coordinate from %f to %f", old_x, x);
     gst_structure_set (structure, "pointer_x", G_TYPE_DOUBLE, x, NULL);
   }
-  if (gst_structure_get_double (structure, "pointer_y", &old_y)) {
-    y = old_y;
+  if (gst_structure_get_double (structure, "pointer_y", &y)) {
+    double old_y = y;
 
     if (y >= result.y && y <= (result.y + result.h)) {
       y -= result.y;
@@ -2222,8 +2225,6 @@ gst_dfbvideosink_finalize (GObject * object)
   if (dfbvideosink->setup) {
     gst_dfbvideosink_cleanup (dfbvideosink);
   }
-
-  G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static void
@@ -2268,9 +2269,7 @@ gst_dfbvideosink_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_set_details_simple (element_class, "DirectFB video sink",
-      "Sink/Video",
-      "A DirectFB based videosink", "Julien Moutte <julien@moutte.net>");
+  gst_element_class_set_details (element_class, &gst_dfbvideosink_details);
 
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_dfbvideosink_sink_template_factory));
@@ -2367,11 +2366,6 @@ gst_dfbvideosink_get_type (void)
       NULL,
       NULL,
     };
-    static const GInterfaceInfo colorbalance_info = {
-      (GInterfaceInitFunc) gst_dfbvideosink_colorbalance_init,
-      NULL,
-      NULL,
-    };
 
     dfbvideosink_type = g_type_register_static (GST_TYPE_VIDEO_SINK,
         "GstDfbVideoSink", &dfbvideosink_info, 0);
@@ -2380,8 +2374,6 @@ gst_dfbvideosink_get_type (void)
         GST_TYPE_IMPLEMENTS_INTERFACE, &iface_info);
     g_type_add_interface_static (dfbvideosink_type, GST_TYPE_NAVIGATION,
         &navigation_info);
-    g_type_add_interface_static (dfbvideosink_type, GST_TYPE_COLOR_BALANCE,
-        &colorbalance_info);
   }
 
   return dfbvideosink_type;
